@@ -34,11 +34,25 @@ class Build(JenkinsBase):
         return self._data["result"]
 
     def get_revision(self):
+        vcs = self._data['changeSet']['kind'] or 'git'
+        return getattr(self, '_get_%s_rev' % vcs, lambda: None)()
+
+    def _get_svn_rev(self):
         maxRevision = 0
         for repoPathSet in self._data["changeSet"]["revisions"]:
             maxRevision = max(repoPathSet["revision"], maxRevision)
         return maxRevision
 
+    def _get_git_rev(self):
+        for item in self._data['actions']:
+            branch = item.get('buildsByBranchName')
+            head = branch and branch.get('origin/HEAD')
+            if head:
+                return head['revision']['SHA1']
+
+    def _get_hg_rev(self):
+        return [x['mercurialNodeName'] for x in self._data['actions'] if 'mercurialNodeName' in x][0]
+
     def get_duration(self):
         return self._data["duration"]
 
@@ -50,7 +64,7 @@ class Build(JenkinsBase):
             del af, url
 
     def get_artifact_dict(self):
-        return dict( (a.filename, a) for a in self.get_artifacts() )
+        return dict( (a.url[len(a.build.baseurl + "artifact/"):], a) for a in self.get_artifacts() )
 
     def get_upstream_job_name(self):
         """
@@ -82,6 +96,17 @@ class Build(JenkinsBase):
         except KeyError:
             return None
 
+    def get_upstream_build(self):
+        """
+        Get the upstream build if it exist, None otherwise
+        :return Build or None
+        """
+        upstream_job = self.get_upstream_job()
+        if upstream_job:
+            return upstream_job.get_build(self.get_upstream_build_number())
+        else:
+            return None
+
     def get_master_job_name(self):
         """
         Get the master job name if it exist, None otherwise
@@ -112,6 +137,68 @@ class Build(JenkinsBase):
         except KeyError:
             return None
 
+    def get_master_build(self):
+        """
+        Get the master build if it exist, None otherwise
+        :return Build or None
+        """
+        master_job = self.get_master_job()
+        if master_job:
+            return master_job.get_build(self.get_master_build_number())
+        else:
+            return None
+
+    def get_downstream_jobs(self):
+        """
+        Get the downstream jobs for this build
+        :return List of jobs or None
+        """
+        downstream_jobs_names = self.job.get_downstream_job_names()
+        fingerprint_data = self.get_data("%s?depth=2&tree=fingerprint[usage[name]]" % self.python_api_url(self.baseurl))
+        downstream_jobs = []
+        try:
+            fingerprints = fingerprint_data['fingerprint'][0]
+            for f in fingerprints['usage']:
+                if f['name'] in downstream_jobs_names:
+                    downstream_jobs.append(self.get_jenkins_obj().get_job(f['name']))
+            return downstream_jobs
+        except (IndexError, KeyError):
+            return None
+
+    def get_downstream_job_names(self):
+        """
+        Get the downstream job names for this build
+        :return List of string or None
+        """
+        downstream_jobs_names = self.job.get_downstream_job_names()
+        fingerprint_data = self.get_data("%s?depth=2&tree=fingerprint[usage[name]]" % self.python_api_url(self.baseurl))
+        downstream_names = []
+        try:
+            fingerprints = fingerprint_data['fingerprint'][0]
+            for f in fingerprints['usage']:
+                if f['name'] in downstream_jobs_names:
+                    downstream_names.append(f['name'])
+            return downstream_names
+        except (IndexError, KeyError):
+            return None
+
+    def get_downstream_builds(self):
+        """
+        Get the downstream builds for this build
+        :return List of Build or None
+        """
+        downstream_jobs_names = self.job.get_downstream_job_names()
+        fingerprint_data = self.get_data("%s?depth=2&tree=fingerprint[usage[name,ranges[ranges[end,start]]]]" % self.python_api_url(self.baseurl))
+        downstream_builds = []
+        try:
+            fingerprints = fingerprint_data['fingerprint'][0]
+            for f in fingerprints['usage']:
+                if f['name'] in downstream_jobs_names:
+                    downstream_builds.append(self.get_jenkins_obj().get_job(f['name']).get_build(f['ranges']['ranges'][0]['start']))
+            return downstream_builds
+        except (IndexError, KeyError):
+            return None
+
     def is_running( self ):
         """
         Return a bool if running.
@@ -153,8 +240,6 @@ class Build(JenkinsBase):
         if self.STR_TOTALCOUNT not in self.get_actions():
             raise NoResults( "%s does not have any published results" % str(self) )
         buildstatus = self.get_status()
-        if buildstatus in [ STATUS_FAIL, RESULTSTATUS_FAILURE, STATUS_ABORTED ]:
-            raise FailedNoResults( self.STR_TPL_NOTESTS_ERR % ( str(self), buildstatus ) )
         if not self.get_actions()[self.STR_TOTALCOUNT]:
             raise NoResults( self.STR_TPL_NOTESTS_ERR % ( str(self), buildstatus ) )
         obj_results = ResultSet( result_url, build=self )
