@@ -4,7 +4,7 @@ from jenkinsapi.job import Job
 from jenkinsapi.view import View
 from jenkinsapi.node import Node
 from jenkinsapi.exceptions import UnknownJob, NotAuthorized
-from utils.urlopener import mkurlopener, mkopener, NoAuto302Handler
+from utils.urlopener import mkurlopener, mkkrbopener, mkopener, NoAuto302Handler
 import logging
 import time
 import urllib2
@@ -22,7 +22,7 @@ class Jenkins(JenkinsBase):
     """
     Represents a jenkins environment.
     """
-    def __init__(self, baseurl, username=None, password=None, proxyhost=None, proxyport=None, proxyuser=None, proxypass=None, formauth=False):
+    def __init__(self, baseurl, username=None, password=None, proxyhost=None, proxyport=None, proxyuser=None, proxypass=None, formauth=False, krbauth=False):
         """
 
         :param baseurl: baseurl for jenkins instance including port, str
@@ -40,7 +40,13 @@ class Jenkins(JenkinsBase):
         self.proxyport = proxyport
         self.proxyuser = proxyuser
         self.proxypass = proxypass
-        JenkinsBase.__init__(self, baseurl, formauth=formauth)
+        JenkinsBase.__init__(self, baseurl, formauth=formauth, krbauth=krbauth)
+
+    def _clone(self):
+        return Jenkins(self.baseurl, username=self.username,
+                       password=self.password, proxyhost=self.proxyhost,
+                       proxyport=self.proxyport, proxyuser=self.proxyuser,
+                       proxypass=self.proxypass, formauth=self.formauth, krbauth=self.krbauth)
 
     def get_proxy_auth(self):
         return self.proxyhost, self.proxyport, self.proxyuser, self.proxypass
@@ -58,6 +64,8 @@ class Jenkins(JenkinsBase):
     def get_opener(self):
         if self.formauth:
             return self.get_login_opener()
+        if self.krbauth:
+            return self.get_krb_opener()
         return mkurlopener(*self.get_auth())
 
     def get_login_opener(self):
@@ -69,6 +77,9 @@ class Jenkins(JenkinsBase):
             hdrs.append(urllib2.HTTPCookieProcessor(mcj))
         return mkopener(*hdrs)
 
+    def get_krb_opener(self):
+        return mkkrbopener(self.baseurl)
+
     def login(self):
         formdata = dict(j_username=self.username, j_password=self.password,
                         remember_me=True, form='/')
@@ -117,6 +128,14 @@ class Jenkins(JenkinsBase):
         for info in self._data["jobs"]:
             yield info["name"], Job(info["url"], info["name"], jenkins_obj=self)
 
+    def get_jobs_info(self):
+        """
+        Get the jobs information
+        :return url, name
+        """
+        for info in self._data["jobs"]:
+            yield info["url"], info["name"]
+
     def get_jobs_list(self):
         """
         return jobs dict,'name:url'
@@ -142,9 +161,24 @@ class Jenkins(JenkinsBase):
         """
         return jobname in self.get_jobs_list()
 
+    def create_job(self, jobname, config):
+        """
+        Create a job
+        :param jobname: name of new job, str
+        :param config: configuration of new job, xml
+        :return: new Job obj
+        """
+        headers = {'Content-Type': 'text/xml'}
+        qs = urllib.urlencode({'name': jobname})
+        url = urlparse.urljoin(self.baseurl, "createItem?%s" % qs)
+        request = urllib2.Request(url, config, headers)
+        self.post_data(request, None)
+        newjk = self._clone()
+        return newjk.get_job(jobname)
+
     def copy_job(self, jobname, newjobname):
         """
-        Copy a job 
+        Copy a job
         :param jobname: name of a exist job, str
         :param newjobname: name of new job, str
         :return: new Job obj
@@ -154,7 +188,8 @@ class Jenkins(JenkinsBase):
                                'from': jobname})
         copy_job_url = urlparse.urljoin(self.baseurl, "createItem?%s" % qs)
         self.post_data(copy_job_url, '')
-        return Jenkins(self.baseurl).get_job(newjobname)
+        newjk = self._clone()
+        return newjk.get_job(newjobname)
 
     def delete_job(self, jobname):
         """
@@ -162,9 +197,23 @@ class Jenkins(JenkinsBase):
         :param jobname: name of a exist job, str
         :return: new jenkins_obj
         """
-        delete_job_url = urlparse.urljoin(Jenkins(self.baseurl).get_job(jobname).baseurl, "sdoDelete" )
+        delete_job_url = urlparse.urljoin(self._clone().get_job(jobname).baseurl, "doDelete" )
         self.post_data(delete_job_url, '')
-        return Jenkins(self.baseurl)
+        newjk = self._clone()
+        return newjk
+
+    def rename_job(self, jobname, newjobname):
+        """
+        Rename a job
+        :param jobname: name of a exist job, str
+        :param newjobname: name of new job, str
+        :return: new Job obj
+        """
+        qs = urllib.urlencode({'newName': newjobname})
+        rename_job_url = urlparse.urljoin(self._clone().get_job(jobname).baseurl, "doRename?%s" % qs)
+        self.post_data(rename_job_url, '')
+        newjk = self._clone()
+        return newjk.get_job(newjobname)
 
     def iteritems(self):
         return self.get_jobs()
@@ -211,7 +260,8 @@ class Jenkins(JenkinsBase):
     def delete_view_by_url(self, str_url):
         url = "%s/doDelete" %str_url
         self.post_data(url, '')
-        return Jenkins(self.baseurl)
+        newjk = self._clone()
+        return newjk
 
     def create_view(self, str_view_name, people=None):
         """
@@ -219,18 +269,14 @@ class Jenkins(JenkinsBase):
         :param str_view_name: name of new view, str
         :return: new view obj
         """
-        
-        if people:
-            url = urlparse.urljoin(self.baseurl, "user/%s/my-views" % people)
-        else:
-            url = self.baseurl
+        url = urlparse.urljoin(self.baseurl, "user/%s/my-views/" % people) if people else self.baseurl
         qs = urllib.urlencode({'value': str_view_name})
         viewExistsCheck_url = urlparse.urljoin(url, "viewExistsCheck?%s" % qs)
         fn_urlopen = self.get_jenkins_obj().get_opener()
         try:
             r = fn_urlopen(viewExistsCheck_url).read()
         except urllib2.HTTPError, e:
-            log.debug("Error reading %s" % url)
+            log.debug("Error reading %s" % viewExistsCheck_url)
             log.exception(e)
             raise
         """<div/>"""
@@ -245,9 +291,9 @@ class Jenkins(JenkinsBase):
                 createView_url = urlparse.urljoin(url, "createView")
                 result = self.post_data(createView_url, params)
             except urllib2.HTTPError, e:
-                log.debug("Error post_data %s" % url)
+                log.debug("Error post_data %s" % createView_url)
                 log.exception(e)
-            return urlparse.urljoin(url, "view/%s" % str_view_name)
+            return urlparse.urljoin(url, "view/%s/" % str_view_name)
 
     def __getitem__(self, jobname):
         """
@@ -255,9 +301,9 @@ class Jenkins(JenkinsBase):
         :param jobname: name of job, str
         :return: Job obj
         """
-        for name, job in self.get_jobs():
+        for url, name in self.get_jobs_info():
             if name == jobname:
-                return job
+                return Job(url, name, jenkins_obj=self)
         raise UnknownJob(jobname)
 
     def get_node_dict(self):
@@ -270,7 +316,7 @@ class Jenkins(JenkinsBase):
 
     def get_node(self, nodename):
         """Get a node object for a specific node"""
-        node_url = self.python_api_url(self.get_node_url(nodename))
+        node_url = self.get_node_url(nodename)
         return Node(node_url, nodename, jenkins_obj=self)
 
     def get_node_url(self, nodename=""):
@@ -319,7 +365,7 @@ class Jenkins(JenkinsBase):
         :param exclusive: tied to specific job, boolean
         :return: node obj
         """
-        NODE_TYPE   = 'jenkins.slaves.DumbSlave$DescriptorImpl'
+        NODE_TYPE   = 'hudson.slaves.DumbSlave$DescriptorImpl'
         MODE = 'NORMAL'
         if self.has_node(name):
             return Node(nodename=name, baseurl=self.get_node_url(nodename=name), jenkins_obj=self)
@@ -336,16 +382,13 @@ class Jenkins(JenkinsBase):
                 'labelString'     : labels,
                 'mode'            : MODE,
                 'type'            : NODE_TYPE,
-                'retentionStrategy' : { 'stapler-class'  : 'jenkins.slaves.RetentionStrategy$Always' },
+                'retentionStrategy' : { 'stapler-class'  : 'hudson.slaves.RetentionStrategy$Always' },
                 'nodeProperties'    : { 'stapler-class-bag' : 'true' },
-                'launcher'          : { 'stapler-class' : 'jenkins.slaves.JNLPLauncher' }
+                'launcher'          : { 'stapler-class' : 'hudson.slaves.JNLPLauncher' }
             })
         }
-        url = "%(nodeurl)s/doCreateItem?%(params)s" % {
-            'nodeurl': self.get_node_url(),
-            'params': urllib.urlencode(params)
-        }
-        print url
+        url = self.get_node_url() + "doCreateItem?%s" % urllib.urlencode(params)
+
         fn_urlopen = self.get_jenkins_obj().get_opener()
         try:
             fn_urlopen(url).read()
