@@ -23,17 +23,19 @@ class JenkinsBase(object):
     def __str__(self):
         raise NotImplemented
 
-    def __init__(self, baseurl, poll=True):
+    def __init__(self, baseurl, poll=True, formauth=False, krbauth=False):
         """
         Initialize a jenkins connection
         """
         self.baseurl = baseurl
-        if poll:
+        self.formauth = formauth
+        self.krbauth = krbauth
+        if poll and not self.formauth:
             try:
                 self.poll()
             except urllib2.HTTPError, hte:
                 log.exception(hte)
-                log.warn( "Failed to conenct to %s" % baseurl )
+                log.warn( "Failed to connect to %s" % baseurl )
                 raise
 
     def poll(self):
@@ -67,6 +69,8 @@ class JenkinsBase(object):
             stream = fn_urlopen(url)
             result = eval(stream.read())
         except urllib2.HTTPError, e:
+            if e.code == 404:
+                raise
             log.warn("Error reading %s" % url)
             log.exception(e)
             raise
@@ -74,8 +78,8 @@ class JenkinsBase(object):
 
     def post_data(self, url, content):
         try:
-            request = urllib2.Request(url, content)
-            result = urllib2.urlopen(request).read().strip()
+            urlopen = self.get_jenkins_obj().get_opener()
+            result = urlopen(url, data=content).read().strip()
         except urllib2.HTTPError, e:
             log.warn("Error post data %s" % url)
             log.exception(e)
@@ -91,4 +95,4 @@ class JenkinsBase(object):
             log.debug( "Error reading %s" % url )
             log.exception(e)
             raise
-        return html_result
\ No newline at end of file
+        return html_result
